@@ -111,7 +111,7 @@ def imaging_distortion(input_model, reference_files):
     """
     Create the "detector" to "v2v3" transform for the MIRI Imager.
 
-    1. Filter dependent shift in (x,y) (!with an oposite
+    1. Filter dependent shift in (x,y) (!with an opposite
        sign to that delivered by the IT) (uses the "filteroffset" ref file)
     2. Apply MI (uses "distortion" ref file)
     3. Apply Ai and BI matrices (uses "distortion" ref file)
@@ -514,7 +514,7 @@ def abl_to_v2v3l(input_model, reference_files):
         label_mapper is LabelMapperDict()
         {channel_wave_range (): channel_number}
         selector is {channel_number: ab2v2 & ab2v3}
-    bacward_transform
+    backward_transform
       RegionsSelector
         label_mapper is LabelMapperDict()
         {channel_wave_range (): channel_number}
