@@ -4,6 +4,7 @@ import asdf
 from astropy import coordinates as coord
 from astropy import units as u
 from astropy.modeling.models import Const1D, Mapping, Identity, Shift
+from astropy.modeling.bounding_box import CompoundBoundingBox
 import gwcs.coordinate_frames as cf
 from gwcs import wcs
 
@@ -88,6 +89,28 @@ def niriss_soss_set_input(model, order_number):
     return wcs.WCS(pipeline)
 
 
+def _niriss_order_bounding_box(input_model, order):
+    import numpy as np
+    bbox_y = np.array([-0.5, input_model.meta.subarray.ysize - 0.5])
+    bbox_x = np.array([-0.5, input_model.meta.subarray.xsize - 0.5])
+
+    if order == 1:
+        return (tuple(bbox_y), tuple(bbox_x))
+    elif order == 2:
+        return (tuple(bbox_y), tuple(bbox_x))
+    elif order == 3:
+        return (tuple(bbox_y), tuple(bbox_x))
+    else:
+        raise ValueError(f'Invalid spectral order: {order} provided. Spectral order must be 1, 2, or 3.')
+
+
+def niriss_bounding_box(input_model):
+    bbox = {(order,): _niriss_order_bounding_box(input_model, order)
+            for order in [1, 2, 3]}
+    model = input_model.meta.wcs.forward_transform
+    return CompoundBoundingBox.validate(model, bbox, slice_args=[('spectral_order', True)], order='F')
+
+
 def niriss_soss(input_model, reference_files):
     """
     The NIRISS SOSS WCS pipeline.
@@ -135,32 +158,28 @@ def niriss_soss(input_model, reference_files):
     try:
         # We'd like to open this file as a DataModel, so we can consolidate
         # the S3 URI handling to one place.  The S3-related code here can
-        # be removed once that has been changed.
+        # be removed once we have a specwcs DataModel subclass.
         if s3_utils.is_s3_uri(reference_files['specwcs']):
-            wl = asdf.open(s3_utils.get_object(reference_files['specwcs']))
+            bytesio_or_path = s3_utils.get_object(reference_files['specwcs'])
         else:
-            wl = asdf.open(reference_files['specwcs'])
-        with wl:
-            wl1 = wl.tree[1].copy()
-            wl2 = wl.tree[2].copy()
-            wl3 = wl.tree[3].copy()
-    except Exception:
-        raise IOError('Error reading wavelength correction from {}'.format(reference_files['specwcs']))
+            bytesio_or_path = reference_files['specwcs']
+        with asdf.open(bytesio_or_path) as af:
+            wl1 = af.tree[1].copy()
+            wl2 = af.tree[2].copy()
+            wl3 = af.tree[3].copy()
+    except Exception as e:
+        raise IOError(f"Error reading wavelength correction from {reference_files['specwcs']}") from e
 
-    try:
-        velosys = input_model.meta.wcsinfo.velosys
-    except AttributeError:
-        pass
-    else:
-        if velosys is not None:
-            velocity_corr = velocity_correction(input_model.meta.wcsinfo.velosys)
-            wl1 = wl1 | velocity_corr
-            wl2 = wl2 | velocity_corr
-            wl2 = wl3 | velocity_corr
-            log.info("Applied Barycentric velocity correction: {}".format(velocity_corr[1].amplitude.value))
+    velosys = input_model.meta.wcsinfo.velosys
+    if velosys is not None:
+        velocity_corr = velocity_correction(velosys)
+        wl1 = wl1 | velocity_corr
+        wl2 = wl2 | velocity_corr
+        wl2 = wl3 | velocity_corr
+        log.info(f"Applied Barycentric velocity correction: {velocity_corr[1].amplitude.value}")
 
     # Reverse the order of inputs passed to Tabular because it's in python order in modeling.
-    # Consider changing it in modelng ?
+    # Consider changing it in modeling ?
     cm_order1 = (Mapping((0, 1, 1, 0)) |
                  (Const1D(target_ra) & Const1D(target_dec) & wl1)
                  ).rename('Order1')
