@@ -139,8 +139,8 @@ def compute_scale(wcs: WCS, fiducial: Union[tuple, np.ndarray],
     yscale = np.abs(coords[0].separation(coords[2]).value)
 
     if pscale_ratio is not None:
-        xscale = xscale * pscale_ratio
-        yscale = yscale * pscale_ratio
+        xscale *= pscale_ratio
+        yscale *= pscale_ratio
 
     if spectral:
         # Assuming scale doesn't change with wavelength
@@ -194,7 +194,8 @@ def calc_rotation_matrix(roll_ref: float, v3i_yang: float, vparity: int = 1) ->
 
 
 def wcs_from_footprints(dmodels, refmodel=None, transform=None, bounding_box=None,
-                        pscale_ratio=None):
+                        pscale_ratio=None, pscale=None, rotation=None,
+                        shape=None, crpix=None, crval=None):
     """
     Create a WCS from a list of input data models.
 
@@ -205,7 +206,7 @@ def wcs_from_footprints(dmodels, refmodel=None, transform=None, bounding_box=Non
     If ``refmodel`` is None, the first WCS object in the list is considered
     a reference. The output coordinate frame and projection (for celestial frames)
     is taken from ``refmodel``.
-    If ``transform`` is not suplied, a compound transform is created using
+    If ``transform`` is not supplied, a compound transform is created using
     CDELTs and PC.
     If ``bounding_box`` is not supplied, the bounding_box of the new WCS is computed
     from bounding_box of all input WCSs.
@@ -225,8 +226,33 @@ def wcs_from_footprints(dmodels, refmodel=None, transform=None, bounding_box=Non
     bounding_box : tuple, optional
         Bounding_box of the new WCS.
         If not supplied it is computed from the bounding_box of all inputs.
-    pscale_ratio : float, optional
-        Ratio of input to output pixel scale.
+    pscale_ratio : float, None, optional
+        Ratio of input to output pixel scale. Ignored when either
+        ``transform`` or ``pscale`` are provided.
+    pscale : float, None, optional
+        Absolute pixel scale in degrees. When provided, overrides
+        ``pscale_ratio``. Ignored when ``transform`` is provided.
+    rotation : float, None, optional
+        Position angle of output image’s Y-axis relative to North.
+        A value of 0.0 would orient the final output image to be North up.
+        The default of `None` specifies that the images will not be rotated,
+        but will instead be resampled in the default orientation for the camera
+        with the x and y axes of the resampled image corresponding
+        approximately to the detector axes. Ignored when ``transform`` is
+        provided.
+    shape : tuple of int, None, optional
+        Shape of the image (data array) using ``numpy.ndarray`` convention
+        (``ny`` first and ``nx`` second). This value will be assigned to
+        ``pixel_shape`` and ``array_shape`` properties of the returned
+        WCS object.
+    crpix : tuple of float, None, optional
+        Position of the reference pixel in the image array.  If ``crpix`` is not
+        specified, it will be set to the center of the bounding box of the
+        returned WCS object.
+    crval : tuple of float, None, optional
+        Right ascension and declination of the reference pixel. Automatically
+        computed if not provided.
+
     """
     bb = bounding_box
     wcslist = [im.meta.wcs for im in dmodels]
@@ -244,6 +270,14 @@ def wcs_from_footprints(dmodels, refmodel=None, transform=None, bounding_box=Non
             raise TypeError("Expected refmodel to be an instance of DataModel.")
 
     fiducial = compute_fiducial(wcslist, bb)
+    if crval is not None:
+        # overwrite spatial axes with user-provided CRVAL:
+        i = 0
+        for k, axt in enumerate(wcslist[0].output_frame.axes_type):
+            if axt == 'SPATIAL':
+                fiducial[k] = crval[i]
+                i += 1
+
     ref_fiducial = compute_fiducial([refmodel.meta.wcs])
 
     prj = astmodels.Pix2Sky_TAN()
@@ -256,21 +290,26 @@ def wcs_from_footprints(dmodels, refmodel=None, transform=None, bounding_box=Non
         # Need to put the rotation matrix (List[float, float, float, float])
         # returned from calc_rotation_matrix into the correct shape for
         # constructing the transformation
-        roll_ref = np.deg2rad(refmodel.meta.wcsinfo.roll_ref)
         v3yangle = np.deg2rad(refmodel.meta.wcsinfo.v3yangle)
         vparity = refmodel.meta.wcsinfo.vparity
+        if rotation is None:
+            roll_ref = np.deg2rad(refmodel.meta.wcsinfo.roll_ref)
+        else:
+            roll_ref = np.deg2rad(rotation) + (vparity * v3yangle)
+
         pc = np.reshape(
             calc_rotation_matrix(roll_ref, v3yangle, vparity=vparity),
             (2, 2)
         )
 
-        rotation = astmodels.AffineTransformation2D(pc)
+        rotation = astmodels.AffineTransformation2D(pc, name='pc_rotation_matrix')
         transform.append(rotation)
 
         if sky_axes:
-            scale = compute_scale(refmodel.meta.wcs, ref_fiducial,
-                                  pscale_ratio=pscale_ratio)
-            transform.append(astmodels.Scale(scale) & astmodels.Scale(scale))
+            if not pscale:
+                pscale = compute_scale(refmodel.meta.wcs, ref_fiducial,
+                                       pscale_ratio=pscale_ratio)
+            transform.append(astmodels.Scale(pscale, name='cdelt1') & astmodels.Scale(pscale, name='cdelt2'))
 
         if transform:
             transform = functools.reduce(lambda x, y: x | y, transform)
@@ -292,14 +331,23 @@ def wcs_from_footprints(dmodels, refmodel=None, transform=None, bounding_box=Non
         output_bounding_box.append((axis_min, axis_max))
 
     output_bounding_box = tuple(output_bounding_box)
-    ax1, ax2 = np.array(output_bounding_box)[sky_axes]
-    offset1 = (ax1[1] - ax1[0]) / 2
-    offset2 = (ax2[1] - ax2[0]) / 2
-    offsets = astmodels.Shift(-offset1) & astmodels.Shift(-offset2)
+    if crpix is None:
+        ax1, ax2 = np.array(output_bounding_box)[sky_axes]
+        offset1 = (ax1[1] - ax1[0]) / 2
+        offset2 = (ax2[1] - ax2[0]) / 2
+    else:
+        offset1, offset2 = crpix
+    offsets = astmodels.Shift(-offset1, name='crpix1') & astmodels.Shift(-offset2, name='crpix2')
 
     wnew.insert_transform('detector', offsets, after=True)
     wnew.bounding_box = output_bounding_box
 
+    if shape is None:
+        shape = [int(axs[1] - axs[0] + 0.5) for axs in output_bounding_box[::-1]]
+
+    wnew.pixel_shape = shape[::-1]
+    wnew.array_shape = shape
+
     return wnew
 
 
@@ -412,7 +460,7 @@ def subarray_transform(input_model):
     tr_xstart = astmodels.Identity(1)
     tr_ystart = astmodels.Identity(1)
 
-    # These quanities are 1-based
+    # These quantities are 1-based
     xstart = input_model.meta.subarray.xstart
     ystart = input_model.meta.subarray.ystart
 
@@ -551,7 +599,7 @@ def create_grism_bbox(input_model,
     wfss_extract_half_height : int, optional
         Cross-dispersion extraction half height in pixels, WFSS mode.
         Overwrites the computed extraction height in ``GrismObject.order_bounding.``
-        If ``None``, it's computed from the segementation map,
+        If ``None``, it's computed from the segmentation map,
         using the min and max wavelength for each of the orders that
         are available.
     wavelength_range : dict, optional
@@ -688,10 +736,10 @@ def _create_grism_bbox(input_model, mmag_extract=99.0,
                     # didn't call this bounding box code. So I think it's safe to leave the subarray
                     # subtraction out, i.e. do not subtract x/ystart.
 
-                    xmin = int(np.min(xstack))
-                    xmax = int(np.max(xstack))
-                    ymin = int(np.min(ystack))
-                    ymax = int(np.max(ystack))
+                    xmin = int(np.nanmin(xstack))
+                    xmax = int(np.nanmax(xstack))
+                    ymin = int(np.nanmin(ystack))
+                    ymax = int(np.nanmax(ystack))
 
                     if wfss_extract_half_height is not None and not obj.is_extended:
                         if input_model.meta.wcsinfo.dispersion_direction == 2:
@@ -718,13 +766,34 @@ def _create_grism_bbox(input_model, mmag_extract=99.0,
                     exclude = False
                     ispartial = False
 
+                    # Here we check to ensure that the extraction region `pts`
+                    # has at least two pixels of width in the dispersion
+                    # direction, and one in the cross-dispersed direction when
+                    # placed into the subarray extent.
                     pts = np.array([[ymin, xmin], [ymax, xmax]])
-                    ll = np.array([0, 0])
-                    ur = np.array([input_model.meta.subarray.ysize, input_model.meta.subarray.xsize])
-                    inidx = np.all(np.logical_and(ll <= pts, pts <= ur), axis=1)
-                    contained = len(pts[inidx])
+                    subarr_extent = np.array([[0, 0],
+                                             [input_model.meta.subarray.ysize - 1,
+                                              input_model.meta.subarray.xsize - 1]])
 
-                    if contained == 0:
+                    if input_model.meta.wcsinfo.dispersion_direction == 1:
+                        # X-axis is dispersion direction
+                        disp_col = 1
+                        xdisp_col = 0
+                    else:
+                        # Y-axis is dispersion direction
+                        disp_col = 0
+                        xdisp_col = 1
+
+                    dispaxis_check = (pts[1, disp_col] - subarr_extent[0, disp_col] > 0) and \
+                                     (subarr_extent[1, disp_col] - pts[0, disp_col] > 0)
+                    xdispaxis_check = (pts[1, xdisp_col] - subarr_extent[0, xdisp_col] >= 0) and \
+                                      (subarr_extent[1, xdisp_col] - pts[0, xdisp_col] >= 0)
+
+                    contained = dispaxis_check and xdispaxis_check
+
+                    inidx = np.all(np.logical_and(subarr_extent[0] <= pts, pts <= subarr_extent[1]), axis=1)
+
+                    if not contained:
                         exclude = True
                         log.info("Excluding off-image object: {}, order {}".format(obj.label, order))
                     elif contained >= 1:
@@ -1074,7 +1143,7 @@ def velocity_correction(velosys):
     """
     correction = (1 / (1 + velosys / c.value))
     model = astmodels.Identity(1) * astmodels.Const1D(correction, name="velocity_correction")
-    model.inverse = astmodels.Identity(1) / astmodels.Const1D(correction, name="inv_vel_correciton")
+    model.inverse = astmodels.Identity(1) / astmodels.Const1D(correction, name="inv_vel_correction")
 
     return model
 
@@ -1117,3 +1186,29 @@ def wrap_ra(ravalues):
         ravalues = ravalues_wrap.tolist()
 
     return ravalues_wrap
+
+
+def in_ifu_slice(slice_wcs, ra, dec, lam):
+    """
+    Given RA, DEC and LAM return the x, y positions within a slice.
+
+    Parameters
+    ----------
+    slice_wcs : `~gwcs.WCS`
+        Slice WCS object.
+    ra, dec, lam : float, ndarray
+        Physical Coordinates.
+
+    Returns
+    -------
+    x, y : float, ndarray
+        x, y locations within the slice.
+    """
+    slicer2world = slice_wcs.get_transform('slicer', 'world')
+    slx, sly, sllam = slicer2world.inverse(ra, dec, lam)
+
+    # Compute the slice X coordinate using the center of the slit.
+    SLX, _, _ = slice_wcs.get_transform('slit_frame', 'slicer')(0, 0, 2e-6)
+    onslice_ind = np.isclose(slx, SLX,atol=5e-4)
+
+    return onslice_ind
