@@ -19,7 +19,7 @@ PRODUCT_NAME_DEFAULT = 'undefined'
 _ASN_NAME_TEMPLATE_STAMP = 'jw{program}-{acid}_{stamp}_{type}_{sequence:03d}_asn'
 _ASN_NAME_TEMPLATE = 'jw{program}-{acid}_{type}_{sequence:03d}_asn'
 
-# Acquistions and Confirmation images
+# Acquisition and Confirmation images
 ACQ_EXP_TYPES = (
     'mir_tacq',
     'mir_taconfirm',
@@ -158,7 +158,7 @@ NRS_FSS_VALID_OPTICAL_PATHS = (
     ('s200b1', 'g140m', 'f070lp', 'nrs2'),
 )
 
-# Key that uniquely identfies members.
+# Key that uniquely identifies members.
 MEMBER_KEY = 'expname'
 
 # Non-specified values found in DMS Association Pools
@@ -333,7 +333,7 @@ class DMSBaseMixin(ACIDMixin):
             Exposure type. Can be one of
 
                 - 'science': Item contains science data
-                - 'target_aquisition': Item contains target acquisition data.
+                - 'target_acquisition': Item contains target acquisition data.
                 - 'autoflat': NIRSpec AUTOFLAT
                 - 'autowave': NIRSpec AUTOWAVE
                 - 'psf': PSF
@@ -400,7 +400,7 @@ class DMSBaseMixin(ACIDMixin):
         is_item_tso : bool
             Item represents a TSO exposure.
         """
-        # If not a science exposure, such as target aquisitions,
+        # If not a science exposure, such as target acquisitions,
         # then other TSO indicators do not apply.
         if item['pntgtype'] != 'science':
             return False
@@ -754,7 +754,7 @@ def get_exposure_type(item, default='science', association=None):
         Exposure type. Can be one of
 
         - 'science': Item contains science data
-        - 'target_aquisition': Item contains target acquisition data.
+        - 'target_acquisition': Item contains target acquisition data.
         - 'autoflat': NIRSpec AUTOFLAT
         - 'autowave': NIRSpec AUTOWAVE
         - 'psf': PSF
@@ -892,3 +892,48 @@ def nrsifu_valid_detector(item):
 
     # Nothing has matched. Not valid.
     return False
+
+
+def nrslamp_valid_detector(item):
+    """Check that a grating/lamp combo can appear on the detector"""
+    try:
+        _, detector = item_getattr(item, ['detector'])
+        _, grating = item_getattr(item, ['grating'])
+        _, opmode = item_getattr(item, ['opmode'])
+    except KeyError:
+        return False
+
+    if opmode in ['msaspec']:
+        # All settings can result in data on both detectors,
+        # depending on which MSA shutters are open
+        return True
+
+    elif opmode in ['ifu']:
+        # Need lamp value for IFU mode
+        try:
+            _, lamp = item_getattr(item, ['lamp'])
+        except KeyError:
+            return False
+
+        # Just a checklist of paths:
+        if grating in ['g395h', 'g235h']:
+            # long-wave, high-res gratings result in data on both detectors
+            return True
+        elif grating in ['g395m', 'g235m', 'g140m'] and detector == 'nrs1':
+            # all medium-res gratings result in data only on NRS1 detector
+            return True
+        elif grating == 'prism' and detector == 'nrs1':
+            # prism results in data only on NRS1 detector
+            return True
+        elif grating == 'g140h':
+            # short-wave, high-res grating results in data on both detectors,
+            # except when lamp FLAT4 is in use (no data on NRS2)
+            if not (detector == 'nrs2' and lamp == 'flat4'):
+                return True
+
+    elif opmode in ['fixedslit']:
+        # All slits illuminated by lamps, regardless of grating or subarray
+        return True
+
+    # Nothing has matched. Not valid.
+    return False
