@@ -13,6 +13,7 @@ from jwst.associations.lib.dms_base import (
     item_getattr,
     nrsfss_valid_detector,
     nrsifu_valid_detector,
+    nrslamp_valid_detector,
 )
 from jwst.associations.lib.member import Member
 from jwst.associations.lib.process_list import ProcessList
@@ -101,6 +102,22 @@ class Asn_Lv2ImageNonScience(
             Constraint_Base(),
             Constraint_Image_Nonscience(),
             Constraint_Single_Science(self.has_science),
+            Constraint(
+                [
+                    DMSAttrConstraint(
+                        name='dms_note',
+                        sources=['dms_note'],
+                        value=['wfsc_los_jitter'],
+                    ),
+                    DMSAttrConstraint(
+                        name='exp_type',
+                        sources=['exp_type'],
+                        value='nrc_image'
+                    ),
+
+                ],
+                reduce=Constraint.notall
+            )
         ])
 
         # Now check and continue initialization.
@@ -507,6 +524,11 @@ class Asn_Lv2NRSLAMPSpectral(
                 sources=['filter'],
                 value='opaque'
             ),
+            SimpleConstraint(
+                value=True,
+                test=lambda value, item: nrslamp_valid_detector(item),
+                force_unique=False
+            ),
             Constraint(
                 [
                     Constraint(
@@ -576,7 +598,7 @@ class Asn_Lv2WFSS(
     Characteristics:
         - Association type: ``spec2``
         - Pipeline: ``calwebb_spec2``
-        - Mutli-object science exposures
+        - Multi-object science exposures
         - Single science exposure
         - Require a source catalog from processing of the corresponding direct imagery.
     """
@@ -754,7 +776,7 @@ class Asn_Lv2WFSS(
         Notes
         -----
         This is an override for the method in `DMSBaseMixin`.
-        The optical element is retieved from the chosen direct image
+        The optical element is retrieved from the chosen direct image
         found in `self.direct_image`, determined in the `self.finalize`
         method.
         """
@@ -961,7 +983,6 @@ class Asn_Lv2WFSC(
             Constraint_Base(),
             Constraint_Image_Science(),
             Constraint_Single_Science(self.has_science),
-            Constraint_ExtCal(),
             Constraint_WFSC(),
         ])
 
