@@ -129,7 +129,7 @@ class Asn_Lv3AMI(AsnMixin_Science):
 
 @RegistryMarker.rule
 class Asn_Lv3Coron(AsnMixin_Science):
-    """Level 3 Coronography Association
+    """Level 3 Coronagraphy Association
 
     Characteristics:
         - Association type: ``coron3``
@@ -138,7 +138,7 @@ class Asn_Lv3Coron(AsnMixin_Science):
 
     Notes
     -----
-    Coronography is nearly completely defined by the association candidates
+    Coronagraphy is nearly completely defined by the association candidates
     produced by APT.
     Tracking Issues:
 
@@ -294,11 +294,7 @@ class Asn_Lv3MIRMRS(AsnMixin_Spectrum):
             DMSAttrConstraint(
                 name='exp_type',
                 sources=['exp_type'],
-                value=(
-                    'mir_mrs'
-                    '|mir_flatmrs'
-                ),
-                force_unique=False
+                value=('mir_mrs'),
             ),
             Constraint(
                 [
@@ -336,11 +332,7 @@ class Asn_Lv3MIRMRSBackground(AsnMixin_AuxData, AsnMixin_Spectrum):
             DMSAttrConstraint(
                 name='exp_type',
                 sources=['exp_type'],
-                value=(
-                    'mir_mrs'
-                    '|mir_flatmrs'
-                ),
-                force_unique=False
+                value=('mir_mrs'),
             ),
             Constraint(
                 [
@@ -542,7 +534,7 @@ class Asn_Lv3SlitlessSpectral(AsnMixin_Spectrum):
                 ],
                 reduce=Constraint.notany
             ),
-            # Constaint to prevent calibration data from level 3 processing
+            # Constraint to prevent calibration data from level 3 processing
             Constraint(
                 [
                     DMSAttrConstraint(
@@ -657,7 +649,7 @@ class Asn_Lv3SpectralTarget(AsnMixin_Spectrum):
         super(Asn_Lv3SpectralTarget, self).__init__(*args, **kwargs)
 
     def finalize(self):
-        """Finalize assocation
+        """Finalize association
 
         For NRS Fixed-slit, finalization means creating new members for the
         background nods.
@@ -773,11 +765,15 @@ class Asn_Lv3WFSCMB(AsnMixin_Science):
             )
         ])
 
-        # Only valid if two members exist.
+        # Only valid if two members exist and candidate is not a GROUP.
         self.validity.update({
             'has_pair': {
                 'validated': False,
                 'check': self._has_pair
+            },
+            'is_not_group': {
+                'validated': False,
+                'check': self._validate_candidates
             }
         })
 
@@ -833,6 +829,25 @@ class Asn_Lv3WFSCMB(AsnMixin_Science):
 
         return len(self.current_product['members']) == count
 
+    def _validate_candidates(self, member):
+        """Disallow GROUP candidates
+
+        Parameters
+        ----------
+        member : Member
+            Member being added. Ignored.
+
+        Returns
+        -------
+        False if candidate is GROUP.
+        True otherwise.
+        """
+
+        # If a group candidate, reject.
+        if self.acid.type.lower() == 'group':
+            return False
+
+        return True
 
 @RegistryMarker.rule
 class Asn_Lv3WFSSNIS(AsnMixin_Spectrum):
