@@ -34,14 +34,15 @@ class CubeBuildStep (Step):
 
     spec = """
          channel = option('1','2','3','4','all',default='all') # Channel
-         band = option('short','medium','long','all',default='all') # Band
+         band = option('short','medium','long','short-medium','short-long','medium-short', \
+                'medium-long', 'long-short', 'long-medium','all',default='all') # Band
          grating   = option('prism','g140m','g140h','g235m','g235h','g395m','g395h','all',default='all') # Grating
          filter   = option('clear','f100lp','f070lp','f170lp','f290lp','all',default='all') # Filter
          output_type = option('band','channel','grating','multi',default='band') # Type IFUcube to create.
          scale1 = float(default=0.0) # cube sample size to use for axis 1, arc seconds
          scale2 = float(default=0.0) # cube sample size to use for axis 2, arc seconds
          scalew = float(default=0.0) # cube sample size to use for axis 3, microns
-         weighting = option('emsm','msm','miripsf',default = 'emsm') # Type of weighting function
+         weighting = option('emsm','msm','drizzle',default = 'emsm') # Type of weighting function
          coord_system = option('skyalign','world','internal_cal','ifualign',default='skyalign') # Output Coordinate system.
          rois = float(default=0.0) # region of interest spatial size, arc seconds
          roiw = float(default=0.0) # region of interest wavelength size, microns
@@ -49,15 +50,12 @@ class CubeBuildStep (Step):
          wavemin = float(default=None)  # Minimum wavelength to be used in the IFUCube
          wavemax = float(default=None)  # Maximum wavelength to be used in the IFUCube
          single = boolean(default=false) # Internal pipeline option used by mrs_imatch & outlier detection
-         xdebug = integer(default=None) # debug option, x spaxel value to report information on
-         ydebug = integer(default=None) # debug option, y spaxel value to report information on
-         zdebug = integer(default=None) # debug option, z spaxel value to report  information on
          skip_dqflagging = boolean(default=false) # skip setting the DQ plane of the IFU
          search_output_file = boolean(default=false)
          output_use_model = boolean(default=true) # Use filenames in the output models
        """
 
-    reference_file_types = ['cubepar', 'resol']
+    reference_file_types = ['cubepar']
 
 # ________________________________________________________________________________
     def process(self, input):
@@ -107,23 +105,6 @@ class CubeBuildStep (Step):
         if self.roiw != 0.0:
             self.log.info(f'Input Wave ROI size {self.roiw}')
 
-        self.debug_file = None
-        if(self.xdebug is not None and self.ydebug is not None and self.zdebug is not None):
-            self.log.info('Writing debug information for spaxel %i %i %i',
-                          self.xdebug,
-                          self.ydebug,
-                          self.zdebug)
-            self.log.debug('Writing debug information for spaxel %i %i %i',
-                           self.xdebug,
-                           self.ydebug,
-                           self.zdebug)
-            self.xdebug = self.xdebug - 1
-            self.ydebug = self.ydebug - 1
-            self.zdebug = self.zdebug - 1
-            self.debug_file = open('cube_spaxel_info.results', 'w')
-            self.debug_file.write('Writing debug information for spaxel %i %i %i' %
-                                  (self.xdebug, self.ydebug, self.zdebug) + '\n')
-
         # valid coord_system:
         # 1. skyalign (ra dec) (aka world)
         # 2. ifualign (ifu cube aligned with slicer plane/ MRS local coord system)
@@ -138,15 +119,18 @@ class CubeBuildStep (Step):
             self.interpolation = 'area'
 
         # if interpolation is point cloud then weighting can be
-        # 1. MSM: modified shepard method
+        # 1. MSM: modified Shepard method
         # 2. EMSM
-        # 3. miripsf - weighting for MIRI based on PSF and LSF
+
         if self.coord_system == 'skyalign':
             self.interpolation = 'pointcloud'
 
         if self.coord_system == 'ifualign':
             self.interpolation = 'pointcloud'
 
+        if self.weighting == 'drizzle':
+            self.interpolation = 'drizzle'
+
         self.log.info(f'Input interpolation: {self.interpolation}')
         self.log.info(f'Coordinate system to use: {self.coord_system}')
         if self.interpolation == 'pointcloud':
@@ -221,16 +205,6 @@ class CubeBuildStep (Step):
             self.log.warning('No default cube parameters reference file found')
             return
 # ________________________________________________________________________________
-# If miripsf weight is set then set up reference file
-        resol_filename = None
-        if self.weighting == 'miripsf':
-            resol_filename = self.get_reference_file(self.input_models[0], 'resol')
-            self.log.info(f'MIRI resol reference file {resol_filename}')
-            if resol_filename == 'N/A':
-                self.log.warning('No spectral resolution reference file found')
-                self.log.warning('Run again and turn off miripsf')
-                return
-# ________________________________________________________________________________
 # shove the input parameters in to pars to pull out in general cube_build.py
 
         pars = {
@@ -257,11 +231,8 @@ class CubeBuildStep (Step):
             'roiw': self.roiw,
             'wavemin': self.wavemin,
             'wavemax': self.wavemax,
-            'skip_dqflagging': self.skip_dqflagging,
-            'xdebug': self.xdebug,
-            'ydebug': self.ydebug,
-            'zdebug': self.zdebug,
-            'debug_file': self.debug_file}
+            'skip_dqflagging': self.skip_dqflagging}
+
 # ________________________________________________________________________________
 # create an instance of class CubeData
 
@@ -269,12 +240,11 @@ class CubeBuildStep (Step):
             self.input_models,
             self.input_filenames,
             par_filename,
-            resol_filename,
             **pars)
 # ________________________________________________________________________________
 # cubeinfo.setup:
 # read in all the input files, information from cube_pars, read in input data
-# and fill in master_table holding what files are associationed with each
+# and fill in master_table holding what files are associated with each
 # ch/sub-ch or grating/filter.
 # Fill in all_channel, all_subchannel,all_filter, all_grating and instrument
 
@@ -351,9 +321,6 @@ class CubeBuildStep (Step):
                 result, status = cube_result
                 cube_container.append(result)
 
-            if self.debug_file is not None:
-                self.debug_file.close()
-
             # check if cube_build failed
             # **************************
             if status == 1:
@@ -382,23 +349,15 @@ class CubeBuildStep (Step):
 # ******************************************************************************
 
     def read_user_input(self):
-        """Read user input options for channel, subchannel, filter, or grating
+        """Read user input options for channel, subchannel, filter, or grating"""
 
-        Determine if any of the input paramters channel, band, filter or
-        grating have been set. If they have been check fill in input_pars
-        dictionary.
+        # Determine if any of the input parameters channel, band, filter or
+        # grating have been set.
 
-        Parameters
-        ----------
-        none
-
-        Notes
-        ------
-        This routine updates the dictionary self.pars_input with any user
-        provided inputs. In particular it sets pars_input['channel'],
-        pars_input['sub_channel'], pars_input['grating'], and
-        pars_input['filter'] with user provided values.
-        """
+        # This routine updates the dictionary self.pars_input with any user
+        # provided inputs. In particular it sets pars_input['channel'],
+        # pars_input['sub_channel'], pars_input['grating'], and
+        # pars_input['filter'] with user provided values.
 
         valid_channel = ['1', '2', '3', '4', 'all']
         valid_subchannel = ['short', 'medium', 'long', 'all', 'short-medium', 'short-long',
