@@ -31,7 +31,7 @@ HUGE_DIST = 1.e10
 
 
 def ifu_extract1d(input_model, ref_dict, source_type, subtract_background,
-                  bkg_sigma_clip, apcorr_ref_model=None):
+                  bkg_sigma_clip, apcorr_ref_model=None, center_xy=None):
     """Extract a 1-D spectrum from an IFU cube.
 
     Parameters
@@ -92,6 +92,18 @@ def ifu_extract1d(input_model, ref_dict, source_type, subtract_background,
 
     extract_params = get_extract_parameters(ref_dict, bkg_sigma_clip, slitname)
 
+    # If the user supplied extraction center coords,
+    # load them into extract_params for use later.
+    extract_params['x_center'] = None
+    extract_params['y_center'] = None
+    if center_xy is not None:
+        if len(center_xy) == 2:
+            extract_params['x_center'] = int(center_xy[0])
+            extract_params['y_center'] = int(center_xy[1])
+            log.info(f'Using user-supplied x_center={center_xy[0]}, y_center={center_xy[1]}')
+        else:
+            log.warning('Incorrect number of values in center_xy; should be two.')
+
     if subtract_background is not None:
         if subtract_background and source_type == "EXTENDED":
             subtract_background = False
@@ -290,7 +302,7 @@ def get_extract_parameters(ref_dict, bkg_sigma_clip, slitname):
     else:
         log.error("Reference file type %s not recognized",
                   ref_dict['ref_file_type'])
-        raise RuntimeError("extract1d reference file must be JSON or a FITS image.")
+        raise RuntimeError("extract1d reference file must be ASDF, JSON or  FITS image.")
 
     return extract_params
 
@@ -410,24 +422,29 @@ def extract_ifu(input_model, source_type, extract_params):
 
     dq = np.zeros(shape[0], dtype=np.uint32)
 
-    # For an extended target, the entire aperture will be extracted, so
-    # it makes no sense to shift the extraction location.
-    if source_type != "EXTENDED":
+    # If the user supplied extraction center coords, use them and
+    # ignore all other source type and source position values
+    if extract_params['x_center'] is not None:
+        x_center = extract_params['x_center']
+        y_center = extract_params['y_center']
+        locn = None
+
+    # For a point source, try to compute the extraction center
+    # from the source location
+    elif source_type != "EXTENDED":
         ra_targ = input_model.meta.target.ra
         dec_targ = input_model.meta.target.dec
         locn = locn_from_wcs(input_model, ra_targ, dec_targ)
 
         if locn is None or np.isnan(locn[0]):
-            log.warning("Couldn't determine pixel location from WCS, so "
-                        "source offset correction will not be applied.")
-
+            log.warning("Couldn't determine source location from WCS, so "
+                        "extraction region will be centered.")
             x_center = float(shape[-1]) / 2.
             y_center = float(shape[-2]) / 2.
-
         else:
             (x_center, y_center) = locn
             log.info("Using x_center = %g, y_center = %g, based on "
-                     "TARG_RA and TARG_DEC.", x_center, y_center)
+                     "TARG_RA and TARG_DEC", x_center, y_center)
 
     method = extract_params['method']
     subpixels = extract_params['subpixels']
@@ -546,8 +563,6 @@ def extract_ifu(input_model, source_type, extract_params):
                                          method=method, subpixels=subpixels)
 
         aperture_area = float(phot_table['aperture_sum'][0])
-        log.debug("aperture.area = %g; aperture_area = %g",
-                  aperture.area, aperture_area)
 
         if(aperture_area == 0 and aperture.area > 0):
             aperture_area = aperture.area
@@ -557,8 +572,6 @@ def extract_ifu(input_model, source_type, extract_params):
             phot_table = aperture_photometry(temp_weightmap, annulus,
                                              method=method, subpixels=subpixels)
             annulus_area = float(phot_table['aperture_sum'][0])
-            log.debug("annulus.area = %g; annulus_area = %g",
-                      annulus.area, annulus_area)
 
             if(annulus_area == 0 and annulus.area > 0):
                 annulus_area = annulus.area
@@ -767,14 +780,14 @@ def image_extract_ifu(input_model, source_type, extract_params):
     source extraction region, 0 for pixels not to include in source or background,
     and -1 for the background region.
     For SRCTYPE=POINT the source extraction region is defined by pixels in the ref
-    image = 1 and the background region is defiend by pixels in the ref image with
+    image = 1 and the background region is defined by pixels in the ref image with
     -1.
     For SRCTYPE=EXTENDED the extraction region is defined by pixels in the ref image
     =  1 (only the source region is used). The default procedure of using the extract 1d
     asdf reference files extracts the entire region for EXTENDED source data. However,
     if the user supplies the reference image it is assumed they have defined a specific
-    region to be extracted instead of the entire field. At each wavelenght bin sigma
-    clipping is performed on the extration region and is store in the background column of
+    region to be extracted instead of the entire field. At each wavelength bin sigma
+    clipping is performed on the extraction region and is store in the background column of
     spec table to be used in masterbackground subtraction. In the extended source case
     pixels flagged as background (-1) in the reference image are ignored.
 
@@ -944,7 +957,7 @@ def image_extract_ifu(input_model, source_type, extract_params):
     npixels[:] = (temp_weightmap * mask_target).sum(axis=2, dtype=np.float64).sum(axis=1)
     bkg_sigma_clip = extract_params['bkg_sigma_clip']
 
-    # Point Souce data 1. extract background and subtract 2. do not
+    # Point Source data 1. extract background and subtract 2. do not
     if source_type == 'POINT':
         if subtract_background and mask_bkg is not None:
             n_bkg[:] = (temp_weightmap * mask_bkg).sum(axis=2, dtype=np.float64).sum(axis=1)
