@@ -115,7 +115,11 @@ class DataSet():
             if model.meta.instrument.grating is not None:
                 self.grating = model.meta.instrument.grating.upper()
             self.order = None
-            if model.meta.get('wcsinfo.spectral_order') is not None:
+            if (
+                model.meta.hasattr('wcsinfo')
+                and model.meta.wcsinfo.hasattr('spectral_order')
+                and model.meta.wcsinfo.spectral_order is not None
+            ):
                 self.order = model.meta.wcsinfo.spectral_order
             self.pupil = None
             if model.meta.instrument.pupil is not None:
@@ -518,7 +522,6 @@ class DataSet():
         Returns
         -------
         """
-        log.debug('Starting cal_nircam')
         # Handle WFSS data separately from regular imaging
         if (isinstance(self.input, datamodels.MultiSlitModel) and self.exptype == 'NRC_WFSS'):
             # Loop over the WFSS slits, applying the correct photom ref data
