@@ -26,6 +26,7 @@ from ..resample import resample_spec_step
 from ..srctype import srctype_step
 from ..straylight import straylight_step
 from ..wavecorr import wavecorr_step
+from ..wfss_contam import wfss_contam_step
 
 __all__ = ['Spec2Pipeline']
 
@@ -51,7 +52,7 @@ class Spec2Pipeline(Pipeline):
     class_alias = "calwebb_spec2"
 
     spec = """
-        save_bsub = boolean(default=False)        # Save background-subracted science
+        save_bsub = boolean(default=False)        # Save background-subtracted science
         fail_on_exception = boolean(default=True) # Fail if any product fails.
     """
 
@@ -70,6 +71,7 @@ class Spec2Pipeline(Pipeline):
         'fringe': fringe_step.FringeStep,
         'pathloss': pathloss_step.PathLossStep,
         'barshadow': barshadow_step.BarShadowStep,
+        'wfss_contam': wfss_contam_step.WfssContamStep,
         'photom': photom_step.PhotomStep,
         'resample_spec': resample_spec_step.ResampleSpecStep,
         'cube_build': cube_build_step.CubeBuildStep,
@@ -352,8 +354,9 @@ class Spec2Pipeline(Pipeline):
             self.log.debug('Science data does not allow fringe correction. Skipping "fringe".')
             self.fringe.skip = True
 
-        # Apply pathloss correction to NIRSpec and NIRISS SOSS exposures
-        if not self.pathloss.skip and exp_type not in ['NRS_FIXEDSLIT', 'NRS_MSASPEC', 'NRS_IFU', 'NIS_SOSS']:
+        # Apply pathloss correction to MIRI LRS, NIRSpec, and NIRISS SOSS exposures
+        if not self.pathloss.skip and exp_type not in ['MIR_LRS-FIXEDSLIT', 'NRS_FIXEDSLIT',
+                                                       'NRS_MSASPEC', 'NRS_IFU', 'NIS_SOSS']:
             self.log.debug('Science data does not allow pathloss correction. Skipping "pathloss".')
             self.pathloss.skip = True
 
@@ -367,6 +370,11 @@ class Spec2Pipeline(Pipeline):
             self.log.debug('Science data does not allow master background correction. Skipping "master_background".')
             self.master_background.skip = True
 
+        # Apply WFSS contamination correction only to WFSS exposures
+        if not self.wfss_contam.skip and exp_type not in WFSS_TYPES:
+            self.log.debug('Science data does not allow WFSS contamination correction. Skipping "wfss_contam".')
+            self.wfss_contam.skip = True
+
     def _process_grism(self, data):
         """WFSS & Grism processing
 
@@ -379,6 +387,7 @@ class Spec2Pipeline(Pipeline):
         calibrated = self.fringe(calibrated)
         calibrated = self.pathloss(calibrated)
         calibrated = self.barshadow(calibrated)
+        calibrated = self.wfss_contam(calibrated)
         calibrated = self.photom(calibrated)
 
         return calibrated
