@@ -39,3 +39,26 @@ def test_cube_build_single_output(run_cube_build_single_output, output, fitsdiff
     # Compare the results
     diff = FITSDiff(rtdata.output, rtdata.truth, **fitsdiff_default_kwargs)
     assert diff.identical, diff.report()
+
+
+@pytest.mark.bigdata
+def test_cube_build_miri_internal_cal(rtdata, fitsdiff_default_kwargs):
+    """Run cube_build on single file using coord system = internal_cal"""
+    input_file = 'det_image_seq2_MIRIFUSHORT_12SHORTexp1_cal.fits'
+    rtdata.get_data(f"miri/mrs/{input_file}")
+
+    args = [
+        'jwst.cube_build.CubeBuildStep',
+        input_file,
+        '--save_results=true',
+        '--coord_system=internal_cal'
+    ]
+    Step.from_cmdline(args)
+
+    output = input_file.replace('cal', 'ch1-short_internal_s3d')
+    rtdata.output = output
+
+    rtdata.get_truth(f"truth/test_miri_cubebuild/{output}")
+
+    diff = FITSDiff(rtdata.output, rtdata.truth, **fitsdiff_default_kwargs)
+    assert diff.identical, diff.report()
