@@ -1,7 +1,6 @@
 from astropy.io.fits.diff import FITSDiff
 import pytest
 
-from jwst.pipeline.collect_pipeline_cfgs import collect_pipeline_cfgs
 from jwst.stpipe import Step
 
 
@@ -12,14 +11,11 @@ def run_pipeline(jail, rtdata_module):
     """
     rtdata = rtdata_module
 
-    # Get the cfg files
-    collect_pipeline_cfgs("config")
-
     # Get the ASN file and input exposures
     rtdata.get_asn('nirspec/fs/jw93045-o010_20180725t035735_spec3_001_asn.json')
 
     # Run the calwebb_spec3 pipeline; save results from intermediate steps
-    args = ["config/calwebb_spec3.cfg", rtdata.input,
+    args = ["calwebb_spec3", rtdata.input,
             "--steps.outlier_detection.save_results=true",
             "--steps.resample_spec.save_results=true",
             "--steps.extract_1d.save_results=true"]
