@@ -2,6 +2,7 @@
 
 from ..stpipe import Step
 from .. import datamodels
+from ..lib import pipe_utils
 from . import saturation
 
 
@@ -36,7 +37,10 @@ class SaturationStep(Step):
             ref_model = datamodels.SaturationModel(self.ref_name)
 
             # Do the saturation check
-            sat = saturation.do_correction(input_model, ref_model)
+            if pipe_utils.is_irs2(input_model):
+                sat = saturation.irs2_flag_saturation(input_model, ref_model)
+            else:
+                sat = saturation.flag_saturation(input_model, ref_model)
 
             # Close the reference file and update the step status
             ref_model.close()
